@@ -3,9 +3,9 @@
 ## author: Brice Ozenne
 ## created: maj 27 2017 (21:23) 
 ## Version: 
-## last-updated: aug 11 2020 (10:54) 
+## last-updated: Oct 14 2024 (12:59) 
 ##           By: Brice Ozenne
-##     Update #: 1085
+##     Update #: 1110
 #----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -37,7 +37,7 @@
 #' @param new.n the number of new observations.
 #' @param cause the cause of interest.
 #' @param nCause the number of causes.
-#' @param nVar the number of variables that form the linear predictor in each Cox model
+#' @param nVar.lp the number of variables that form the linear predictor in each Cox model
 #' @param export can be "iid" to return the value of the influence function for each observation
 #'                      "se" to return the standard error for a given timepoint
 #' @param diag [logical] when \code{FALSE} the absolute risk/survival for all observations at all times is computed,
@@ -58,7 +58,7 @@
 #' @rdname calcSeCSC
 calcSeCSC <- function(object, cif, hazard, cumhazard, survival, object.time, object.maxtime,
                       eXb, new.LPdata, new.strata, times, surv.type, ls.infoVar,
-                      new.n, cause, nCause, nVar, export, store.iid, diag){
+                      new.n, cause, nCause, nVar.lp, export, store.iid, diag){
 
     status <- strata.num <- NULL ## [:CRANcheck:] data.table
                                         # {{{ influence function for each Cox model
@@ -102,6 +102,10 @@ calcSeCSC <- function(object, cif, hazard, cumhazard, survival, object.time, obj
             rm.list <- FALSE
             factor <- attr(export, "factor")
         }
+        if(any(stats::na.omit(as.double(cif))>1)){
+            stop("Cannot use option \'store.iid\'=\"minimal\" with argument \'product.limit\' = -1. \n")
+        }
+
         grid.strata <- unique(new.strata)
         resCpp <- calcSeMinimalCSC_cpp(seqTau = times,
                                        newSurvival = survival, 
@@ -129,7 +133,7 @@ calcSeCSC <- function(object, cif, hazard, cumhazard, survival, object.time, obj
                                        }),
                                        newdata_index = new.indexStrata,
                                        factor = factor, grid_strata = grid.strata,
-                                       nTau = nTimes, nNewObs = new.n, nSample = object.n, nStrata = new.n.strata, nCause = nCause, p = nVar,
+                                       nTau = nTimes, nNewObs = new.n, nSample = object.n, nStrata = new.n.strata, nCause = nCause, p = nVar.lp,
                                        theCause = cause-1, diag = diag, survtype = (surv.type=="survival"),
                                        exportSE = ("se" %in% export),  exportIF = ("iid" %in% export), exportIFmean = ("average.iid" %in% export),
                                        debug = 0)
@@ -165,19 +169,42 @@ calcSeCSC <- function(object, cif, hazard, cumhazard, survival, object.time, obj
                                   ls_cumhazard = cumhazard,
                                   ls_hazard = hazard[[cause]],
                                   survival = survival,
+                                  cif = cif,
                                   ls_IFcumhazard = lapply(object$models, function(x){x$iid$IFcumhazard}),
                                   ls_IFhazard = object$models[[cause]]$iid$IFhazard,
                                   eXb = eXb,
                                   nJumpTime = nEtimes, JumpMax = object.maxtime,
                                   tau = times, tauIndex = sindex.times, nTau = nTimes,                                  
                                   nObs = object.n,
-                                  theCause = (cause-1), nCause = nCause, hazardType = (surv.type=="hazard"), nVar = nVar,
+                                  theCause = (cause-1), nCause = nCause, hazardType = (surv.type=="hazard"), nVar = nVar.lp,
                                   nNewObs = new.n, strata = new.strata,
                                   exportSE = "se" %in% export, exportIF = "iid" %in% export, exportIFsum = "average.iid" %in% export,
                                   diag = diag)
             if("iid" %in% export){
                 out$iid <- aperm(out$iid, c(2,3,1))
             }
+            if("average.iid" %in% export && !is.null(attr(export,"factor"))){
+                ## calcSeCif2_cpp average ignoring the weights defined in attr(export,"factor")
+                factor <- attr(export, "factor")
+                if(diag){
+                    ## Not necessary due to error message
+                    ## Attribute "factor" of argument 'average.iid' not available when 'iid' is TRUE 
+                }else{
+                    ## May occur when compressing data as it bypass the error message
+                    n.newobs <- dim(out$iid)[3]
+                    out$average.iid <- lapply(1:length(factor), function(iF){ ## iObs <- 1
+                        Reduce("+",lapply(1:n.newobs, function(iObs){
+                            if(nTimes==1){
+                                return(cbind(out$iid[,,iObs]*factor[[1]][iObs,]))
+                            }else{
+                                return(rowMultiply_cpp(out$iid[,,iObs], factor[[1]][iObs,]))
+                            }
+                        }))/n.newobs})
+                    if(length(factor)==1){
+                        out$average.iid <- out$average.iid[[1]]
+                    }
+                }
+            }
 
         }else if("average.iid" %in% export){
 
@@ -265,16 +292,16 @@ calcSeCSC <- function(object, cif, hazard, cumhazard, survival, object.time, obj
                 eXb1_S_eXbj <- vector(mode = "list", length = nCause)
                 eXb1_S_eXbj[all.cause] <- lapply(all.cause, function(iC){colMultiply_cpp(eXb1_S, ieXb[[iC]])})
                 
-                if(nVar[cause]>0){
-                    eXb1_S_X1 <- lapply(1:nVar[cause], function(iP){ colMultiply_cpp(eXb1_S, scale = iX[[cause]][,iP]) })
+                if(nVar.lp[cause]>0){
+                    eXb1_S_X1 <- lapply(1:nVar.lp[cause], function(iP){ colMultiply_cpp(eXb1_S, scale = iX[[cause]][,iP]) })
                 }else{
                     eXb1_S_X1 <- NULL
                 }
          
                 eXb1_S_Xj_eXbj <- vector(mode = "list", length = nCause)
                 for(iCause in all.cause){
-                    if(nVar[iCause]>0){
-                        eXb1_S_Xj_eXbj[[iCause]] <- lapply(1:nVar[iCause], function(iP){ colMultiply_cpp(eXb1_S_eXbj[[iCause]], scale = iX[[iCause]][,iP]) })
+                    if(nVar.lp[iCause]>0){
+                        eXb1_S_Xj_eXbj[[iCause]] <- lapply(1:nVar.lp[iCause], function(iP){ colMultiply_cpp(eXb1_S_eXbj[[iCause]], scale = iX[[iCause]][,iP]) })
                     }
                 }
 
@@ -352,9 +379,45 @@ calcSeCSC <- function(object, cif, hazard, cumhazard, survival, object.time, obj
                                              weight = iVN_time, factor = iMfactor,
                                              nJump = iiN.jump, subsetJump = iiIndex.jump,
                                              nCause = nCause, test_allCause = test_allCause, test_theCause = test_theCause,
-                                             nVar = nVar)
+                                             nVar = nVar.lp)
+
+                        if(any(stats::na.omit(as.double(cif))>=1)){ ## average influence function only among datapoint with cif<1
+                            test.cif1 <- cif>=1
+                            vec.index1 <- apply(test.cif1, MARGIN = 1, FUN = function(iRow){which(iRow)[1]})
+                            index.pattern <- sort(unique(stats::na.omit(vec.index1)))
+                            n.pattern <- length(index.pattern)
+                            vec.index1[is.na(vec.index1)] <- Inf
+
+                            for(iP in 1:n.pattern){ ## iP <- 1
+
+                                iPattern <- index.pattern[iP]
+                                if(iP == n.pattern){
+                                    iIndex.rangePattern <- which(iTime.jump>=times[iPattern])
+                                }else{
+                                    iIndex.rangePattern <- seq(which(iTime.jump>=times[iPattern])[1],utils::tail(which(iTime.jump<times[index.pattern[iP+1]]),1), by = 1)
+                                }
+                                iIndex.keep <- which(vec.index1>iPattern)                                
+                                if(length(iIndex.keep)==0){
+                                    iAIF[,iIndex.rangePattern] <- 0
+                                }else{
+                                    iiIndex.jump2 <- 1:max(iIndex.rangePattern)
+                                    iAIF[,iIndex.rangePattern] <- calcAICcif_R(hazard0_cause = ihazard0_cause,
+                                                                               cumhazard0 = iCumhazard0,
+                                                                               IFhazard0_cause = iIFhazard0_cause,
+                                                                               IFcumhazard0 = iIFCumhazard0,
+                                                                               IFbeta = lapply(object$models,function(iM){iM$iid$IFbeta}),                                         
+                                                                               eXb1_S = eXb1_S[iIndex.keep,,drop=FALSE],
+                                                                               eXb1_S_eXbj = lapply(eXb1_S_eXbj,function(iM){iM[iIndex.keep,,drop=FALSE]}),
+                                                                               eXb1_S_X1 = lapply(eXb1_S_X1,function(iM){iM[iIndex.keep,,drop=FALSE]}),
+                                                                               eXb1_S_Xj_eXbj = lapply(eXb1_S_Xj_eXbj, function(iLs){lapply(iLs,function(iM){iM[iIndex.keep,,drop=FALSE]})}), 
+                                                                               weight = iVN_time[iiIndex.jump2], factor = iMfactor[iIndex.keep,iiIndex.jump2,drop=FALSE],
+                                                                               nJump = length(iiIndex.jump2), subsetJump = iiIndex.jump2,
+                                                                               nCause = nCause, test_allCause = test_allCause, test_theCause = test_theCause,
+                                                                               nVar = nVar.lp)[,iIndex.rangePattern,drop=FALSE]
+                                }
+                            }
+                        }
 
-             
                         ## export
                         if(diag==TRUE){
                             out[[iFactor]][,1] <- out[[iFactor]][,1] + rowSums(iAIF[,iSindexV.times,drop=FALSE])/iN_obs * iPrevalence
